/*
 * Copyright 2018 Analytics Zoo Authors.
 *
 * Licensed under the Apache License, Version 2.0 (the "License");
 * you may not use this file except in compliance with the License.
 * You may obtain a copy of the License at
 *
 *     http://www.apache.org/licenses/LICENSE-2.0
 *
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an "AS IS" BASIS,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 * See the License for the specific language governing permissions and
 * limitations under the License.
 */

package com.intel.analytics.zoo.pipeline.api.keras.layers

import com.intel.analytics.bigdl.tensor.Tensor
import com.intel.analytics.bigdl.utils.Shape
import com.intel.analytics.zoo.pipeline.api.keras.models.Sequential
import com.intel.analytics.zoo.pipeline.api.keras.serializer.ModuleSerializationTest

import scala.util.Random

class SpatialDropout2DSpec extends KerasBaseSpec {

  "SpatialDropout2D NCHW forward and backward" should "work properly" in {
    val seq = Sequential[Float]()
    val layer = SpatialDropout2D[Float](0.5, "th", inputShape = Shape(3, 4, 5))
    seq.add(layer)
    seq.getOutputShape().toSingle().toArray should be (Array(-1, 3, 4, 5))
    val input = Tensor[Float](2, 3, 4, 5).rand()
    val output = seq.forward(input)
    val gradInput = seq.backward(input, output)
  }

  "SpatialDropout2D NHWC forward and backward" should "work properly" in {
    val seq = Sequential[Float]()
    val layer = SpatialDropout2D[Float](0.5, "tf", inputShape = Shape(3, 4, 5))
    seq.add(layer)
    seq.getOutputShape().toSingle().toArray should be (Array(-1, 3, 4, 5))
    val input = Tensor[Float](2, 3, 4, 5).rand()
    val output = seq.forward(input)
    val gradInput = seq.backward(input, output)
  }

}

class SpatialDropout2DSerialTest extends ModuleSerializationTest {
  override def test(): Unit = {
    val layer = SpatialDropout2D[Float](0.5, "tf", inputShape = Shape(3, 64, 64))
    layer.build(Shape(2, 3, 64, 64))
    val input = Tensor[Float](2, 3, 64, 64).apply1(_ => Random.nextFloat())
    runSerializationTest(layer, input)
  }
}
